/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view.AccuracyTests.Enh5;

import com.topcoder.rules.dependencymap.view.Dependency;
import com.topcoder.rules.dependencymap.view.DiagramRenderer;
import com.topcoder.rules.dependencymap.view.RendererException;
import com.topcoder.rules.dependencymap.view.AccuracyTests.BaseLayoutTests;
import com.topcoder.rules.dependencymap.view.AccuracyTests.GraphicsHelper;
import com.topcoder.rules.dependencymap.view.AccuracyTests.MockDiagramRenderer;
import com.topcoder.rules.dependencymap.view.AccuracyTests.MockGraphics;

/**
 * Test the supress blanks enhancement.
 * @author cucu
 */
public class Enh5Tests extends BaseLayoutTests {

    /**
     * Generates an instance of DiagramRenderer that will be used for the tests.
     */
    protected DiagramRenderer createDiagramRenderer() {
        return new MockDiagramRenderer();
    }

    /**
     * Save the expected and the resulting SVG. Please compare them visually.
     */
    protected void compare(String test, GraphicsHelper gh, DiagramRenderer dr) {
        ((MockGraphics)dr.getGraphics(a,1,1)).save("test_files\\enh5\\" + test + "_result");
        gh.getGraphics().save("test_files\\enh5\\" + test);
    }

    /**
     * Test with the sample 1 found in the specification doc.
     */
    public void testSample1() throws RendererException {
        Dependency h = new Dependency("H");
        Dependency i = new Dependency("I");
        Dependency j = new Dependency("J");
        Dependency k = new Dependency("K");
        Dependency l = new Dependency("L");
        Dependency m = new Dependency("M");
        Dependency n = new Dependency("N");
        Dependency o = new Dependency("O");
        Dependency p = new Dependency("P");

        a.addDependant(b);
        a.addDependant(i);
        b.addDependant(c);
        b.addDependant(f);
        b.addDependant(g);
        b.addDependant(h);
        i.addDependant(j);
        i.addDependant(k);
        i.addDependant(l);
        i.addDependant(m);
        c.addDependant(d);
        c.addDependant(e);
        m.addDependant(n);
        m.addDependant(o);
        m.addDependant(p);
        renderer.render(a);

        gh.setShapeSizeTo("A");

        gh.drawShape(false, 6.5, 1, "A");
        gh.drawArrow(6.5, 1, 4, 2);
        gh.drawShape(false, 4, 2, "B");
        gh.drawArrow(4, 2, 2, 3);
        gh.drawShape(false, 2, 3, "C");
        gh.drawArrow(2, 3, 1, 4);
        gh.drawShape(false, 1, 4, "D");
        gh.drawArrow(2, 3, 3, 4);
        gh.drawShape(false, 3, 4, "E");
        gh.drawArrow(4, 2, 4, 3);
        gh.drawShape(false, 4, 3, "F");
        gh.drawArrow(4, 2, 5, 3);
        gh.drawShape(false, 5, 3, "G");
        gh.drawArrow(4, 2, 6, 3);
        gh.drawShape(false, 6, 3, "H");
        gh.drawArrow(6.5, 1, 9, 2);
        gh.drawShape(false, 9, 2, "I");
        gh.drawArrow(9, 2, 7, 3);
        gh.drawShape(false, 7, 3, "J");
        gh.drawArrow(9, 2, 8, 3);
        gh.drawShape(false, 8, 3, "K");
        gh.drawArrow(9, 2, 9, 3);
        gh.drawShape(false, 9, 3, "L");
        gh.drawArrow(9, 2, 11, 3);
        gh.drawShape(false, 11, 3, "M");
        gh.drawArrow(11, 3, 10, 4);
        gh.drawShape(false, 10, 4, "N");
        gh.drawArrow(11, 3, 11, 4);
        gh.drawShape(false, 11, 4, "O");
        gh.drawArrow(11, 3, 12, 4);
        gh.drawShape(false, 12, 4, "P");


        compare("testSample1", gh, diagramRenderer);
    }

    /**
     * Test with the sample 2 found in the specification doc.
     */
    public void testSample2() throws RendererException {
        Dependency h = new Dependency("H");
        Dependency i = new Dependency("I");
        Dependency j = new Dependency("J");
        Dependency k = new Dependency("K");

        a.addDependant(b);
        b.addDependant(c);
        b.addDependant(d);
        b.addDependant(k);
        d.addDependant(e);
        d.addDependant(h);
        d.addDependant(i);
        d.addDependant(j);
        e.addDependant(f);
        e.addDependant(g);
        renderer.render(a);

        gh.setShapeSizeTo("A");

        gh.drawShape(false, 4, 1, "A");
        gh.drawArrow(4, 1, 4, 2);
        gh.drawShape(false, 4, 2, "B");
        gh.drawArrow(4, 2, 1, 3);
        gh.drawShape(false, 1, 3, "C");
        gh.drawArrow(4, 2, 4, 3);
        gh.drawShape(false, 4, 3, "D");
        gh.drawArrow(4, 3, 3, 4);
        gh.drawShape(false, 3, 4, "E");
        gh.drawArrow(3, 4, 2, 5);
        gh.drawShape(false, 2, 5, "F");
        gh.drawArrow(3, 4, 4, 5);
        gh.drawShape(false, 4, 5, "G");
        gh.drawArrow(4, 3, 5, 4);
        gh.drawShape(false, 5, 4, "H");
        gh.drawArrow(4, 3, 6, 4);
        gh.drawShape(false, 6, 4, "I");
        gh.drawArrow(4, 3, 7, 4);
        gh.drawShape(false, 7, 4, "J");
        gh.drawArrow(4, 2, 8, 3);
        gh.drawShape(false, 8, 3, "K");

        compare("testSample2", gh, diagramRenderer);
    }

}
